@@ -24,15 +24,23 @@ import sys
 import shutil
 import logging
 import zc.recipe.egg
-from collective.recipe.omelette.utils import symlink, unlink, islink, rmtree
+from collective.recipe.omelette.utils import symlink, unlink, islink, rmtree, WIN32
 
-def makedirs(target):
+NAMESPACE_STANZA = """# See http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
+try:
+    __import__('pkg_resources').declare_namespace(__name__)
+except ImportError:
+    from pkgutil import extend_path
+    __path__ = extend_path(__path__, __name__)
+"""
+
+def makedirs(target, is_namespace=False):
     """ Similar to os.makedirs, but adds __init__.py files as it goes.  Returns a boolean
         indicating success.
     """
     drive, path = os.path.splitdrive(target)
     parts = path.split(os.path.sep)
-    current = drive + os.path.sep    
+    current = drive + os.path.sep
     for part in parts:
         current = os.path.join(current, part)
         if islink(current):
@@ -42,7 +50,10 @@ def makedirs(target):
             init_filename = os.path.join(current, '__init__.py')
             if not os.path.exists(init_filename):
                 init_file = open(init_filename, 'w')
-                init_file.write("# mushroom")
+                if is_namespace or part == 'Products':
+                    init_file.write(NAMESPACE_STANZA)
+                else:
+                    init_file.write('# mushroom')
                 init_file.close()
     return True
     
@@ -77,36 +88,83 @@ class Recipe(object):
 
     def install(self):
         """Crack the eggs open and mix them together"""
-        
+
         location = self.options['location']
         if os.path.exists(location):
             rmtree(location)
-        
+        os.mkdir(location)
+
         try:
             requirements, ws = self.egg.working_set()
             for dist in ws.by_key.values():
                 project_name =  dist.project_name
-                if  project_name not in self.ignored_eggs:
-                    parts = project_name.split('.')
-                    namespaces = parts[:-1]
-                    package_name = parts[-1]
-                    egg_location = os.path.join(dist.location, *parts)
-                    
-                    link_dir = os.path.join(location, *namespaces)
-                    if not os.path.exists(link_dir):
-                        if not makedirs(link_dir):
-                            self.logger.warn("Warning: (While processing egg %s) Could not create containing directory.  Skipping." % (project_name))
+                if project_name not in self.ignored_eggs:
+                    namespaces = {}
+                    for line in dist._get_metadata('namespace_packages.txt'):
+                        ns = namespaces
+                        for part in line.split('.'):
+                            ns = ns.setdefault(part, {})
+                    top_level = list(dist._get_metadata('top_level.txt'))
+                    native_libs = list(dist._get_metadata('native_libs.txt'))
+                    def create_namespaces(namespaces, ns_base=()):
+                        for k, v in namespaces.iteritems():
+                            ns_parts = ns_base + (k,)
+                            link_dir = os.path.join(location, *ns_parts)
+                            if not os.path.exists(link_dir):
+                                if not makedirs(link_dir, is_namespace=True):
+                                    self.logger.warn("Warning: (While processing egg %s) Could not create namespace directory (%s).  Skipping." % (project_name, link_dir))
+                                    continue
+                            if len(v) > 0:
+                                create_namespaces(v, ns_parts)
+                            else:
+                                egg_ns_dir = os.path.join(dist.location, *ns_parts)
+                                dirs = os.listdir(egg_ns_dir)
+                                for name in dirs:
+                                    if not os.path.isdir(os.path.join(egg_ns_dir, name)):
+                                        continue
+                                    if name.startswith('.'):
+                                        continue
+                                    name_parts = ns_parts + (name,)
+                                    src = os.path.join(dist.location, *name_parts)
+                                    dst = os.path.join(location, *name_parts)
+                                    if os.path.exists(dst):
+                                        continue
+                                    symlink(src, dst)
+                    create_namespaces(namespaces)
+                    for package_name in top_level:
+                        if package_name in namespaces:
+                            # These are processed in create_namespaces
                             continue
-                    link_location = os.path.join(link_dir, package_name)
-                    if not os.path.exists(egg_location):
-                        self.logger.warn("Warning: (While processing egg %s) Egg contents not found at %s.  Skipping." % (project_name, egg_location))
-                        continue
-                    if not os.path.exists(link_location):
-                        symlink(egg_location, link_location)
-                    else:
-                        self.logger.warn("Warning: (While processing egg %s) Link already exists.  Skipping." % project_name)
-                        continue
-                    
+                        else:
+                            if not os.path.isdir(dist.location):
+                                self.logger.warn("Warning: (While processing egg %s) Package '%s' is zipped.  Skipping." % (project_name, package_name))
+                                continue
+                            package_location = os.path.join(dist.location, package_name)
+                            link_location = os.path.join(location, package_name)
+                            # check for single python module
+                            if not os.path.exists(package_location):
+                                package_location = os.path.join(dist.location, package_name+".py")
+                                link_location = os.path.join(location, package_name+".py")
+                            # check for native libs
+                            # XXX - this should use native_libs from above
+                            if not os.path.exists(package_location):
+                                package_location = os.path.join(dist.location, package_name+".so")
+                                link_location = os.path.join(location, package_name+".so")
+                            if not os.path.exists(package_location):
+                                package_location = os.path.join(dist.location, package_name+".dll")
+                                link_location = os.path.join(location, package_name+".dll")
+                            if not os.path.exists(package_location):
+                                self.logger.warn("Warning: (While processing egg %s) Package '%s' not found.  Skipping." % (project_name, package_name))
+                                continue
+                        if not os.path.exists(link_location):
+                            if WIN32 and not os.path.isdir(package_location):
+                                self.logger.warn("Warning: (While processing egg %s) Can't link files on Windows (%s -> %s).  Skipping." % (project_name, package_location, link_location))
+                                continue
+                            symlink(package_location, link_location)
+                        else:
+                            self.logger.warn("Warning: (While processing egg %s) Link already exists (%s -> %s).  Skipping." % (project_name, package_location, link_location))
+                            continue
+
             for package in self.packages:
                 if len(package) == 1:
                     link_name = 'Products/'
@@ -153,7 +211,7 @@ class Recipe(object):
                 else:
                     symlink(package_location, link_location)
         else:
-            print "Warning: Product directory %s not found.  Skipping." % package_dir
+            self.logger.warn("Warning: Product directory %s not found.  Skipping." % package_dir)
         
 def uninstall(name, options):
     location = options.get('location')
